@@ -60,6 +60,8 @@ have_ipv6_connection <- local({
   function(url = ipv6_url()) {
     if (is.null(ok) || myurl != url) {
       myurl <<- url
+      opt <- options(warn = 2)
+      on.exit(options(opt), add = TRUE)
       tryCatch({
         cx <- curl::curl(url)
         open(cx)
@@ -103,7 +105,6 @@ cleanup_process <- function(p) {
   tryCatch(close(p$get_error_connection()), error = function(x) x)
   tryCatch(close(p$get_poll_connection()), error = function(x) x)
   tryCatch(p$kill(), error = function(x) x)
-  gc()
 }
 
 httpbin <- webfakes::new_app_process(
